import { useEffect, useState } from 'react';
import { useTranslation } from 'react-i18next';
import { AccountStatus, AllUser, AllUserResponse } from '@/types/user';
import Header from '@/components/Header';
import ContentDrawer from '@/components/ContentDrawer';
import UserCreate from '@/components/UserCreate';
import { Avatar, Table } from 'antd';
import { TableProps } from 'antd/lib';
import { UserService } from '@/services/userService';
import UserEdit from '@/components/UserEdit';

export default function AccountsPage() {
  const { t } = useTranslation();
  const [openCreate, setOpenCreate] = useState(false);

  const [activeAccount, setActiveAccount] = useState<
    { login: string; id: number; name: string; email: string } | undefined
  >(undefined);

  const showCreateDrawer = () => setOpenCreate(true);
  const closeCreateDrawer = () => {
    setActiveAccount(undefined);
    setOpenCreate(false);
  };
  const [openEdit, setOpenEdit] = useState(false);

  const showEditDrawer = () => setOpenEdit(true);
  const closeEditDrawer = () => {
    setActiveAccount(undefined);
    setOpenEdit(false);
  };

  const [accounts, setAccounts] = useState<AllUserResponse>({
    amountCount: 0,
    amountPages: 0,
    users: [],
    currentPage: 1,
  });

  useEffect(() => {
    async function getUsers() {
      const data = await UserService.getUsers();
      setAccounts(data);
    }

    getUsers();
  }, []);

  const statusColor = {
    ACTIVE: '#27AE60',
    DISABLED: '#606060',
    BLOCKED: '#FF0000',
    DELETED: '#B30000',
  };

  const columns: TableProps<AllUser>['columns'] = [
    {
      title: '#',
      dataIndex: 'id',
      key: 'id',
    },
    {
      title: t('nameLogin'),
      dataIndex: 'nameLogin',
      key: 'nameLogin',
      render: (text, record) => (
        <div
          onClick={() => {
            setActiveAccount({
              login: record.login,
              id: record.id,
              name: record.name,
              email: record.email || '',
            });
            showEditDrawer();
          }}
          style={{
            display: 'flex',
            alignItems: 'center',
            gap: '16px',
            cursor: 'pointer',
          }}
        >
          <div
            style={{
              height: '32px',
              width: '32px',
              display: 'flex',
              alignItems: 'center',
              justifyContent: 'center',
            }}
          >
            <Avatar
              size={32}
              src={`https://gamma.heado.ru/go/ava?name=${record.login}`}
            />
          </div>
          <div style={{ display: 'flex', flexDirection: 'column' }}>
            <div>{record.name}</div>
            <div style={{ color: '#606060' }}>{record.login}</div>
          </div>
        </div>
      ),
    },
    {
      title: 'E-mail',
      dataIndex: 'email',
      key: 'email',
    },
    {
      title: t('tenant'),
      dataIndex: 'bindTenantId',
      key: 'tenant',
    },
    {
      title: t('role'),
      dataIndex: 'role',
      key: 'role',
      render: (text) => <div>{t(text)}</div>,
    },
    {
      title: t('createdAt'),
      dataIndex: 'createdAt',
      key: 'createdAt',
    },
    {
      title: t('status'),
      dataIndex: 'status',
      key: 'status',
      render: (text) => (
        <div style={{ color: statusColor[text as AccountStatus] }}>
          {t(text)}
        </div>
      ),
    },
  ];

  const onTableChange: TableProps<AllUser>['onChange'] = (pagination) => {
    console.log(pagination);
    UserService.getUsers(pagination.current as number, 10).then((data) => {
      setAccounts(data);
    });
  };

  return (
    <>
      <Header
        title={t('accounts')}
        additionalContent={
          <img
            src="./icons/header/add_2.svg"
            alt="add"
            style={{
              height: '18px',
              width: '18px',
              cursor: 'pointer',
            }}
            onClick={showCreateDrawer}
          />
        }
      />
      <Table
        onChange={onTableChange}
        columns={columns}
        dataSource={accounts.users}
        pagination={{
          pageSize: 10,
          current: accounts.currentPage,
          total: accounts.amountCount,
        }}
        rowKey={'id'}
      />

      <ContentDrawer
        open={openCreate}
        closeDrawer={closeCreateDrawer}
        type="create"
      >
        <UserCreate />
      </ContentDrawer>
      <ContentDrawer
        login={activeAccount?.login}
        name={activeAccount?.name}
        email={activeAccount?.email}
        open={openEdit}
        closeDrawer={closeEditDrawer}
        type="edit"
      >
        <UserEdit userId={activeAccount?.id} />
      </ContentDrawer>
    </>
  );
}
